"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.BackupManager = void 0;
const vscode = __importStar(require("vscode"));
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const os = __importStar(require("os"));
class BackupManager {
    static getDefaultBackupFolder() {
        const homeDir = os.homedir();
        switch (os.platform()) {
            case 'win32':
                return path.join(homeDir, 'Documents', 'CodeSnippets');
            case 'darwin':
                return path.join(homeDir, 'Library', 'Application Support', 'CodeSnippets');
            default: // Linux and others
                return path.join(homeDir, '.config', 'codesnippets');
        }
    }
    static async getBackupFolder() {
        const config = vscode.workspace.getConfiguration('snippets');
        let backupFolder = config.get('backupFolder');
        if (!backupFolder) {
            backupFolder = this.getDefaultBackupFolder();
            await config.update('backupFolder', backupFolder, true);
        }
        // Ensure the folder exists
        if (!fs.existsSync(backupFolder)) {
            fs.mkdirSync(backupFolder, { recursive: true });
        }
        return backupFolder;
    }
    static async configureBackupFolder() {
        const currentFolder = await this.getBackupFolder();
        const options = {
            canSelectFiles: false,
            canSelectFolders: true,
            canSelectMany: false,
            defaultUri: vscode.Uri.file(currentFolder),
            openLabel: 'Select Backup Folder'
        };
        const result = await vscode.window.showOpenDialog(options);
        if (result && result[0]) {
            const newFolder = result[0].fsPath;
            await vscode.workspace.getConfiguration('snippets').update('backupFolder', newFolder, true);
            // Ensure the folder exists
            if (!fs.existsSync(newFolder)) {
                fs.mkdirSync(newFolder, { recursive: true });
            }
            vscode.window.showInformationMessage(`Backup folder set to: ${newFolder}`);
        }
    }
    static async backupSnippet(snippet) {
        const backupFolder = await this.getBackupFolder();
        const fileName = `${snippet.name.replace(/[^a-z0-9]/gi, '_')}.${snippet.language || 'txt'}`;
        const filePath = path.join(backupFolder, fileName);
        const content = [
            `// Snippet: ${snippet.name}`,
            `// Language: ${snippet.language || 'plaintext'}`,
            `// Tags: ${(snippet.tags || []).join(', ')}`,
            '// Notes:',
            ...snippet.notes.split('\n').map(line => `// ${line}`),
            '',
            snippet.code
        ].join('\n');
        fs.writeFileSync(filePath, content, 'utf8');
    }
}
exports.BackupManager = BackupManager;
//# sourceMappingURL=BackupManager.js.map