"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.SnippetTreeDataProvider = void 0;
const vscode = __importStar(require("vscode"));
const SnippetTreeItem_1 = require("./SnippetTreeItem");
class SnippetTreeDataProvider {
    constructor(localStorage) {
        this.localStorage = localStorage;
        this._onDidChangeTreeData = new vscode.EventEmitter();
        this.onDidChangeTreeData = this._onDidChangeTreeData.event;
        this.searchQuery = '';
        this.folders = [];
        this.snippets = [];
        this.isLoading = false;
        this.loadData().catch(error => {
            console.error('[DEBUG] Error in initial load:', error);
        });
    }
    async loadData() {
        if (this.isLoading) {
            console.log('[DEBUG] Already loading data, skipping');
            return;
        }
        this.isLoading = true;
        try {
            console.log('[DEBUG] Loading tree data');
            const data = await this.localStorage.getAllData();
            this.folders = data.folders;
            this.snippets = data.snippets;
            console.log('[DEBUG] Loaded data:', {
                folders: this.folders.length,
                snippets: this.snippets.length,
                folderIds: this.folders.map(f => f.id),
                snippetIds: this.snippets.map(s => s.id)
            });
        }
        catch (error) {
            console.error('[DEBUG] Error loading data:', error);
            throw error;
        }
        finally {
            this.isLoading = false;
        }
    }
    async refresh() {
        console.log('[DEBUG] Starting tree view refresh');
        try {
            await this.loadData();
            console.log('[DEBUG] Data loaded, firing refresh event');
            this._onDidChangeTreeData.fire(undefined);
            console.log('[DEBUG] Tree view refresh complete');
        }
        catch (error) {
            console.error('[DEBUG] Error during refresh:', error);
            throw error;
        }
    }
    setSearchQuery(query) {
        this.searchQuery = query.toLowerCase();
        // Update context to show/hide clear search button
        vscode.commands.executeCommand('setContext', 'snippets:hasSearchQuery', !!query);
        this.refresh();
    }
    clearSearch() {
        this.searchQuery = '';
        vscode.commands.executeCommand('setContext', 'snippets:hasSearchQuery', false);
        this.refresh();
    }
    getTreeItem(element) {
        return element;
    }
    // Handle drag and drop
    async handleDrop(target, sources) {
        const snippetData = sources.get('application/vnd.code.tree.snippetsExplorer');
        if (!snippetData) {
            return;
        }
        const sourceItem = JSON.parse(snippetData.value);
        if (sourceItem.type !== 'snippet' || target.type !== 'folder') {
            return;
        }
        // Don't move if the target is the same as the current parent
        if (sourceItem.parentId === target.id) {
            return;
        }
        try {
            const snippet = await this.localStorage.getSnippet(sourceItem.id);
            if (snippet) {
                await this.localStorage.updateSnippet({
                    id: snippet.id,
                    folderId: target.id
                });
                this.refresh();
            }
        }
        catch (error) {
            vscode.window.showErrorMessage('Failed to move snippet: ' + error);
        }
    }
    // Handle drag
    async handleDrag(source, dataTransfer) {
        if (source.length === 1 && source[0].type === 'snippet') {
            dataTransfer.set('application/vnd.code.tree.snippetsExplorer', new vscode.DataTransferItem(JSON.stringify(source[0])));
        }
    }
    snippetMatchesSearch(snippet) {
        if (!this.searchQuery) {
            return true;
        }
        const searchTerms = this.searchQuery.toLowerCase().split(' ');
        return searchTerms.every(term => {
            // Check name match
            if (snippet.name.toLowerCase().includes(term)) {
                return true;
            }
            // Check tags match
            if (snippet.tags && snippet.tags.some(tag => tag.toLowerCase().includes(term))) {
                return true;
            }
            // Check notes match
            if (snippet.notes && snippet.notes.toLowerCase().includes(term)) {
                return true;
            }
            // Check code content match
            if (snippet.code && snippet.code.toLowerCase().includes(term)) {
                return true;
            }
            return false;
        });
    }
    folderMatchesSearch(folder) {
        if (!this.searchQuery) {
            return true;
        }
        return folder.name.toLowerCase().includes(this.searchQuery.toLowerCase());
    }
    async getChildren(element) {
        try {
            if (!element) {
                // Root level - show only root folders
                const rootFolders = this.folders.filter(folder => folder.parentId === null);
                // If searching, also show matching snippets at root level
                if (this.searchQuery) {
                    const matchingSnippets = this.snippets.filter(snippet => this.snippetMatchesSearch(snippet));
                    return [
                        ...rootFolders
                            .filter(folder => this.folderMatchesSearch(folder))
                            .map(folder => new SnippetTreeItem_1.SnippetTreeItem(folder.name, folder.id, 'folder', null)),
                        ...matchingSnippets.map(snippet => new SnippetTreeItem_1.SnippetTreeItem(snippet.name, snippet.id, 'snippet', snippet.folderId, snippet.language))
                    ];
                }
                return rootFolders.map(folder => new SnippetTreeItem_1.SnippetTreeItem(folder.name, folder.id, 'folder', null));
            }
            else if (element.type === 'folder') {
                // Folder level - show subfolders and snippets
                const subfolders = this.folders.filter(folder => folder.parentId === element.id);
                const folderSnippets = this.snippets.filter(snippet => snippet.folderId === element.id);
                return [
                    ...subfolders.map(folder => new SnippetTreeItem_1.SnippetTreeItem(folder.name, folder.id, 'folder', folder.parentId)),
                    ...folderSnippets.map(snippet => new SnippetTreeItem_1.SnippetTreeItem(snippet.name, snippet.id, 'snippet', snippet.folderId, snippet.language))
                ];
            }
            return [];
        }
        catch (error) {
            console.error('[DEBUG] Error getting children:', error);
            return [];
        }
    }
}
exports.SnippetTreeDataProvider = SnippetTreeDataProvider;
//# sourceMappingURL=SnippetTreeDataProvider.js.map