"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.SnippetTreeItem = void 0;
const vscode = __importStar(require("vscode"));
class SnippetTreeItem extends vscode.TreeItem {
    constructor(label, id, type, parentId = null, language) {
        // For folders, use Collapsed state if it has a parentId (subfolder), otherwise Expanded (root folder)
        const collapsibleState = type === 'folder'
            ? (parentId === null ? vscode.TreeItemCollapsibleState.Expanded : vscode.TreeItemCollapsibleState.Collapsed)
            : vscode.TreeItemCollapsibleState.None;
        super(label, collapsibleState);
        this.label = label;
        this.id = id;
        this.type = type;
        this.parentId = parentId;
        this.language = language;
        this.tooltip = type === 'folder' ? `Folder: ${label}` : label;
        this.contextValue = type;
        // Enable drag for snippets and drop for folders
        this.draggable = type === 'snippet';
        this.dropTarget = type === 'folder';
        if (type === 'folder') {
            // Use different icons for root folders and subfolders
            this.iconPath = new vscode.ThemeIcon(parentId === null ? 'folder' : 'folder-opened');
        }
        else {
            // Create a fake file path with the correct extension to get proper file icon
            const extension = this.getLanguageExtension(language || 'plaintext');
            this.resourceUri = vscode.Uri.parse(`file:///fake/path/file.${extension}`);
            // Only pass the necessary properties to avoid circular reference
            this.command = {
                command: 'snippets.openSnippet',
                title: 'Open Snippet',
                arguments: [{
                        id: this.id,
                        name: this.label,
                        type: this.type,
                        language: this.language
                    }]
            };
        }
    }
    getLanguageExtension(language) {
        // Map of languages to their file extensions
        const extensionMap = {
            'javascript': 'js',
            'typescript': 'ts',
            'python': 'py',
            'java': 'java',
            'csharp': 'cs',
            'cpp': 'cpp',
            'c': 'c',
            'go': 'go',
            'rust': 'rs',
            'php': 'php',
            'ruby': 'rb',
            'html': 'html',
            'css': 'css',
            'scss': 'scss',
            'sass': 'sass',
            'less': 'less',
            'json': 'json',
            'xml': 'xml',
            'yaml': 'yml',
            'markdown': 'md',
            'shell': 'sh',
            'bash': 'sh',
            'powershell': 'ps1',
            'sql': 'sql',
            'dockerfile': 'dockerfile',
            'plaintext': 'txt'
        };
        return extensionMap[language.toLowerCase()] || 'txt';
    }
}
exports.SnippetTreeItem = SnippetTreeItem;
//# sourceMappingURL=SnippetTreeItem.js.map