"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.LocalStorage = void 0;
const vscode = __importStar(require("vscode"));
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const os = __importStar(require("os"));
class LocalStorage {
    constructor() {
        this.initialized = false;
        this.storagePath = path.join(os.homedir(), '.vscode', 'snippets');
        this.initializeStorage().catch(error => {
            console.error('Failed to initialize storage:', error);
        });
    }
    async initializeStorage() {
        try {
            // Create storage directory if it doesn't exist
            await fs.promises.mkdir(this.storagePath, { recursive: true });
            // Create folders.json if it doesn't exist
            const foldersPath = path.join(this.storagePath, 'folders.json');
            if (!await this.fileExists(foldersPath)) {
                await fs.promises.writeFile(foldersPath, '[]');
            }
            // Create snippets.json if it doesn't exist
            const snippetsPath = path.join(this.storagePath, 'snippets.json');
            if (!await this.fileExists(snippetsPath)) {
                await fs.promises.writeFile(snippetsPath, '[]');
            }
            this.initialized = true;
        }
        catch (error) {
            console.error('Error initializing storage:', error);
            throw error;
        }
    }
    async fileExists(filePath) {
        try {
            await fs.promises.access(filePath);
            return true;
        }
        catch {
            return false;
        }
    }
    async waitForInitialization() {
        if (!this.initialized) {
            await this.initializeStorage();
        }
    }
    async getFoldersData() {
        await this.waitForInitialization();
        const foldersPath = path.join(this.storagePath, 'folders.json');
        console.log('[DEBUG] Reading folders from:', foldersPath);
        const data = await fs.promises.readFile(foldersPath, 'utf8');
        console.log('[DEBUG] Raw folders data:', data);
        const folders = JSON.parse(data);
        console.log('[DEBUG] Parsed folders:', {
            count: folders.length,
            folders: folders
        });
        return folders;
    }
    async getSnippetsData() {
        await this.waitForInitialization();
        const snippetsPath = path.join(this.storagePath, 'snippets.json');
        console.log('[DEBUG] Reading snippets from:', snippetsPath);
        const data = await fs.promises.readFile(snippetsPath, 'utf8');
        console.log('[DEBUG] Raw snippets data:', data);
        const snippets = JSON.parse(data);
        console.log('[DEBUG] Parsed snippets:', {
            count: snippets.length,
            snippets: snippets
        });
        return snippets;
    }
    async saveFoldersData(folders) {
        await this.waitForInitialization();
        console.log('[DEBUG] Saving folders to storage:', {
            count: folders.length,
            folders: folders
        });
        const foldersPath = path.join(this.storagePath, 'folders.json');
        await fs.promises.writeFile(foldersPath, JSON.stringify(folders, null, 2));
        console.log('[DEBUG] Folders saved to:', foldersPath);
        // Verify the file was written correctly
        try {
            const savedContent = await fs.promises.readFile(foldersPath, 'utf8');
            const savedFolders = JSON.parse(savedContent);
            console.log('[DEBUG] Verified folders file content:', {
                path: foldersPath,
                content: savedContent,
                parsed: savedFolders,
                count: savedFolders.length
            });
        }
        catch (error) {
            console.error('[DEBUG] Error verifying folders file:', error);
        }
    }
    async saveSnippetsData(snippets) {
        await this.waitForInitialization();
        console.log('[DEBUG] Saving snippets to storage:', {
            count: snippets.length,
            snippets: snippets
        });
        const snippetsPath = path.join(this.storagePath, 'snippets.json');
        await fs.promises.writeFile(snippetsPath, JSON.stringify(snippets, null, 2));
        console.log('[DEBUG] Snippets saved to:', snippetsPath);
        // Verify the file was written correctly
        try {
            const savedContent = await fs.promises.readFile(snippetsPath, 'utf8');
            const savedSnippets = JSON.parse(savedContent);
            console.log('[DEBUG] Verified snippets file content:', {
                path: snippetsPath,
                content: savedContent,
                parsed: savedSnippets,
                count: savedSnippets.length
            });
        }
        catch (error) {
            console.error('[DEBUG] Error verifying snippets file:', error);
        }
    }
    // Method to get data from backup file without syncing
    async getBackupData() {
        const backupFolder = vscode.workspace.getConfiguration('snippets').get('backupFolder');
        if (!backupFolder) {
            throw new Error('Backup folder not configured');
        }
        try {
            const backupPath = path.join(backupFolder, 'snippets.json');
            if (!await this.fileExists(backupPath)) {
                return null;
            }
            const backupContent = await fs.promises.readFile(backupPath, 'utf8');
            const backupData = JSON.parse(backupContent);
            if (!backupData.data || !Array.isArray(backupData.data)) {
                throw new Error('Invalid backup file format');
            }
            // Separate into folders and snippets
            const folders = [];
            const snippets = [];
            backupData.data.forEach((item) => {
                if (item.type === 'folder') {
                    const { type, ...folderData } = item;
                    folders.push(folderData);
                }
                else {
                    snippets.push(item);
                }
            });
            return {
                folders,
                snippets,
                timestamp: backupData.timestamp
            };
        }
        catch (error) {
            console.error('[DEBUG] Error reading backup:', error);
            throw error;
        }
    }
    // Method to sync from backup file
    async syncFromBackup() {
        const backupData = await this.getBackupData();
        if (!backupData) {
            throw new Error('No backup data found');
        }
        // Get current data
        const currentData = await this.getAllData();
        const currentItems = new Map();
        [...currentData.folders, ...currentData.snippets].forEach(item => {
            currentItems.set(item.id, item);
        });
        // Merge with backup data, keeping newer versions
        const mergedItems = new Map(currentItems);
        [...backupData.folders, ...backupData.snippets].forEach(item => {
            const current = currentItems.get(item.id);
            if (!current || item.lastModified > current.lastModified) {
                mergedItems.set(item.id, item);
            }
        });
        // Separate into folders and snippets
        const folders = [];
        const snippets = [];
        mergedItems.forEach(item => {
            if ('type' in item) {
                const { type, ...folderData } = item;
                folders.push(folderData);
            }
            else {
                snippets.push(item);
            }
        });
        // Save merged data
        await this.syncData({ folders, snippets });
    }
    async getFolders() {
        return this.getFoldersData();
    }
    async getSubFolders(parentId) {
        const folders = await this.getFoldersData();
        return folders.filter(f => f.parentId === parentId);
    }
    async getRootFolders() {
        const folders = await this.getFoldersData();
        return folders.filter(f => f.parentId === null);
    }
    async getSnippets() {
        return this.getSnippetsData();
    }
    async getAllSnippets() {
        return this.getSnippetsData();
    }
    async addFolder(name, parentId = null, type = 'primary') {
        await this.waitForInitialization();
        const newFolder = {
            id: Date.now().toString(),
            name,
            parentId,
            type,
            lastModified: Date.now()
        };
        const folders = await this.getFolders();
        folders.push(newFolder);
        await this.saveFoldersData(folders);
        await this.updateBackupFile({ folders, snippets: await this.getSnippetsData() });
        return newFolder;
    }
    async addSnippet(snippet) {
        await this.waitForInitialization();
        const newSnippet = {
            id: Date.now().toString(),
            lastModified: Date.now(),
            ...snippet
        };
        const snippets = await this.getSnippets();
        snippets.push(newSnippet);
        await this.saveSnippetsData(snippets);
        await this.updateBackupFile({ folders: await this.getFoldersData(), snippets });
        return newSnippet;
    }
    async deleteFolder(id) {
        await this.waitForInitialization();
        const folders = await this.getFolders();
        const snippets = await this.getSnippets();
        // Remove the folder
        const updatedFolders = folders.filter(folder => folder.id !== id);
        // Remove all snippets in the folder
        const updatedSnippets = snippets.filter(snippet => snippet.folderId !== id);
        await this.saveFoldersData(updatedFolders);
        await this.saveSnippetsData(updatedSnippets);
        await this.updateBackupFile({ folders: updatedFolders, snippets: updatedSnippets });
    }
    async deleteSnippet(id) {
        await this.waitForInitialization();
        const snippets = await this.getSnippets();
        const updatedSnippets = snippets.filter(snippet => snippet.id !== id);
        await this.saveSnippetsData(updatedSnippets);
        await this.updateBackupFile({ folders: await this.getFoldersData(), snippets: updatedSnippets });
    }
    async updateSnippet(update) {
        const snippets = await this.getSnippetsData();
        const snippetIndex = snippets.findIndex(s => s.id === update.id);
        if (snippetIndex !== -1) {
            snippets[snippetIndex] = {
                ...snippets[snippetIndex],
                ...update,
                lastModified: Date.now()
            };
            await this.saveSnippetsData(snippets);
            await this.updateBackupFile({ folders: await this.getFoldersData(), snippets });
        }
    }
    async getSnippet(snippetId) {
        const snippets = await this.getSnippetsData();
        return snippets.find(s => s.id === snippetId);
    }
    async getAllData() {
        console.log('[DEBUG] Getting all data');
        const data = {
            folders: await this.getFoldersData(),
            snippets: await this.getSnippetsData()
        };
        console.log('[DEBUG] Retrieved data:', {
            folders: data.folders.length,
            snippets: data.snippets.length,
            folderDetails: data.folders,
            snippetDetails: data.snippets
        });
        return data;
    }
    async updateBackupFile(data) {
        const backupFolder = vscode.workspace.getConfiguration('snippets').get('backupFolder');
        if (!backupFolder) {
            console.log('[DEBUG] No backup folder configured, skipping backup update');
            return;
        }
        try {
            // Create backup folder if it doesn't exist
            await fs.promises.mkdir(backupFolder, { recursive: true });
            const backupPath = path.join(backupFolder, 'snippets.json');
            console.log('[DEBUG] Updating backup file at:', backupPath);
            // Convert to backup format - ensure it matches the export format exactly
            const backupData = {
                version: "1.0",
                timestamp: new Date().toISOString(),
                data: [
                    ...data.folders.map(folder => ({
                        id: folder.id,
                        name: folder.name,
                        parentId: folder.parentId,
                        type: 'folder',
                        lastModified: folder.lastModified || Date.now()
                    })),
                    ...data.snippets.map(snippet => ({
                        id: snippet.id,
                        name: snippet.name,
                        folderId: snippet.folderId,
                        code: snippet.code || '',
                        language: snippet.language || 'plaintext',
                        notes: snippet.notes || '',
                        tags: snippet.tags || [],
                        lastModified: snippet.lastModified || Date.now()
                    }))
                ]
            };
            // Write to backup file
            await fs.promises.writeFile(backupPath, JSON.stringify(backupData, null, 2));
            console.log('[DEBUG] Backup file updated successfully with data:', backupData);
            // Verify the backup was written correctly
            try {
                const verifyContent = await fs.promises.readFile(backupPath, 'utf8');
                const verifyData = JSON.parse(verifyContent);
                console.log('[DEBUG] Verified backup file content:', {
                    version: verifyData.version,
                    timestamp: verifyData.timestamp,
                    itemCount: verifyData.data.length,
                    path: backupPath
                });
            }
            catch (verifyError) {
                console.error('[DEBUG] Error verifying backup file:', verifyError);
            }
        }
        catch (error) {
            console.error('[DEBUG] Failed to update backup file:', error);
            // Show error to user since this is important for sync
            vscode.window.showErrorMessage(`Failed to update backup file: ${error}`);
        }
    }
    async syncData(data) {
        console.log('[DEBUG] Starting data sync with:', {
            folders: data.folders.length,
            snippets: data.snippets.length,
            folderDetails: data.folders,
            snippetDetails: data.snippets
        });
        await this.waitForInitialization();
        await this.saveFoldersData(data.folders);
        await this.saveSnippetsData(data.snippets);
        // Update backup file after successful save
        await this.updateBackupFile(data);
        console.log('[DEBUG] Data sync completed');
    }
    async renameFolder(folderId, newName) {
        const folders = await this.getFoldersData();
        const folderIndex = folders.findIndex(f => f.id === folderId);
        if (folderIndex !== -1) {
            folders[folderIndex] = {
                ...folders[folderIndex],
                name: newName,
                lastModified: Date.now()
            };
            await this.saveFoldersData(folders);
            await this.updateBackupFile({ folders, snippets: await this.getSnippetsData() });
        }
    }
    async renameSnippet(snippetId, newName) {
        const snippets = await this.getSnippetsData();
        const snippetIndex = snippets.findIndex(s => s.id === snippetId);
        if (snippetIndex !== -1) {
            snippets[snippetIndex] = {
                ...snippets[snippetIndex],
                name: newName,
                lastModified: Date.now()
            };
            await this.saveSnippetsData(snippets);
            await this.updateBackupFile({ folders: await this.getFoldersData(), snippets });
        }
    }
    async exportData() {
        try {
            const data = await this.getAllData();
            const exportData = {
                version: "1.0",
                timestamp: new Date().toISOString(),
                data: {
                    folders: data.folders.map(folder => ({
                        id: folder.id,
                        name: folder.name,
                        type: folder.type,
                        parentId: folder.parentId,
                        lastModified: folder.lastModified || Date.now()
                    })),
                    snippets: data.snippets.map(snippet => ({
                        id: snippet.id,
                        name: snippet.name,
                        code: snippet.code,
                        notes: snippet.notes || "",
                        folderId: snippet.folderId,
                        language: snippet.language || "plaintext",
                        lastModified: snippet.lastModified || Date.now()
                    }))
                }
            };
            return JSON.stringify(exportData, null, 2);
        }
        catch (error) {
            console.error('Error exporting data:', error);
            throw new Error(`Failed to export data: ${error.message}`);
        }
    }
    async importData(jsonData) {
        try {
            const importedData = JSON.parse(jsonData);
            // Validate the imported data structure
            if (!importedData.version || !importedData.data) {
                throw new Error('Invalid import file format');
            }
            // Version compatibility check
            if (importedData.version !== "1.0") {
                console.log(`Warning: Importing data from version ${importedData.version}`);
            }
            const { folders, snippets } = importedData.data;
            // Validate folders
            if (!Array.isArray(folders)) {
                throw new Error('Invalid folders data');
            }
            // Validate snippets
            if (!Array.isArray(snippets)) {
                throw new Error('Invalid snippets data');
            }
            // Validate each folder has required fields
            folders.forEach((folder, index) => {
                if (!folder.id || typeof folder.id !== 'string' ||
                    !folder.name || typeof folder.name !== 'string' ||
                    !folder.type || (folder.type !== 'primary' && folder.type !== 'secondary') ||
                    (folder.parentId !== null && typeof folder.parentId !== 'string')) {
                    throw new Error(`Invalid folder data at index ${index}`);
                }
                // Ensure lastModified exists
                if (!folder.lastModified) {
                    folder.lastModified = Date.now();
                }
            });
            // Validate each snippet has required fields
            snippets.forEach((snippet, index) => {
                if (!snippet.id || typeof snippet.id !== 'string' ||
                    !snippet.name || typeof snippet.name !== 'string' ||
                    typeof snippet.code !== 'string' || // code can be empty but must be string
                    !snippet.folderId || typeof snippet.folderId !== 'string') {
                    throw new Error(`Invalid snippet data at index ${index}`);
                }
                // Ensure lastModified exists
                if (!snippet.lastModified) {
                    snippet.lastModified = Date.now();
                }
            });
            // Create a map of existing folders and snippets
            const currentData = await this.getAllData();
            const existingFolders = new Map(currentData.folders.map(f => [f.id, f]));
            const existingSnippets = new Map(currentData.snippets.map(s => [s.id, s]));
            // Merge folders
            const mergedFolders = new Map();
            // Add existing folders
            existingFolders.forEach((folder, id) => {
                mergedFolders.set(id, {
                    ...folder,
                    lastModified: folder.lastModified || Date.now()
                });
            });
            // Add/update imported folders
            folders.forEach((folder) => {
                const existingFolder = mergedFolders.get(folder.id);
                if (!existingFolder || (folder.lastModified > existingFolder.lastModified)) {
                    mergedFolders.set(folder.id, {
                        ...folder,
                        lastModified: folder.lastModified
                    });
                }
            });
            // Merge snippets
            const mergedSnippets = new Map();
            // Add existing snippets
            existingSnippets.forEach((snippet, id) => {
                mergedSnippets.set(id, {
                    ...snippet,
                    lastModified: snippet.lastModified || Date.now()
                });
            });
            // Add/update imported snippets
            snippets.forEach((snippet) => {
                const existingSnippet = mergedSnippets.get(snippet.id);
                if (!existingSnippet || (snippet.lastModified > existingSnippet.lastModified)) {
                    mergedSnippets.set(snippet.id, {
                        ...snippet,
                        lastModified: snippet.lastModified
                    });
                }
            });
            // Convert maps back to arrays
            const finalData = {
                folders: Array.from(mergedFolders.values()),
                snippets: Array.from(mergedSnippets.values())
            };
            // Save the merged data
            await this.syncData(finalData);
            console.log('Import completed successfully:', {
                folders: finalData.folders.length,
                snippets: finalData.snippets.length
            });
        }
        catch (error) {
            console.error('Error importing data:', error);
            throw new Error(`Failed to import data: ${error.message}`);
        }
    }
}
exports.LocalStorage = LocalStorage;
//# sourceMappingURL=LocalStorage.js.map